<?php

declare(strict_types=1);

namespace Drupal\iconify_field\Twig\Extension;

use Drupal\Component\Utility\Html;
use Drupal\Core\Render\Markup;
use Iconify\IconsJSON\Finder;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

/**
 * IconifyField Twig extension.
 */
class IconifyField extends AbstractExtension {

  /**
   * IconifyField constructor.
   */
  public function __construct() {
    // This method exists to eliminete a PHPCS error
    // about PHP4 style constructors.
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new TwigFunction('iconify_field', $this->iconifyField(...)),
    ];
  }

  /**
   * Get an SVG from an iconify field name.
   *
   * @param string $field
   *   The field to get the icon from.
   *
   * @return string
   *   The SVG icon.
   */
  public function iconifyField(?string $field) {
    $fallback = Markup::create(
      sprintf(
        '<span class="iconify">%s</span>',

        // XSS is bad, m'kay?
        Html::escape($field ?? ''),
      )
    );

    if (!$field || strpos($field, ':') === FALSE) {
      return $fallback;
    }

    [$collection_name, $icon_name] = explode(':', $field);

    $collection_path = Finder::locate($collection_name);

    if (!file_exists($collection_path)) {
      return $fallback;
    }

    $collection = json_decode(file_get_contents($collection_path), TRUE);

    if (!json_last_error() === JSON_ERROR_NONE || !isset($collection['icons'][$icon_name])) {
      return $fallback;
    }

    $collection_width = $collection['width'];
    $collection_height = $collection['height'];
    $icon_width = $collection['icons'][$icon_name]['width'] ?? $collection_width;

    $svg_prefix = sprintf(
      '<svg xmlns="http://www.w3.org/2000/svg" class="iconify" width="%s" height="%s" viewbox="0 0 %s %s">',
      round(($icon_width / $collection_width) * 100) / 100 . 'em',
      '1em',
      $icon_width,
      $collection_height,
    );

    return Markup::create(
      $svg_prefix .
      $collection['icons'][$icon_name]['body'] .
      '</svg>'
    );
  }

}
